import { ParticleOutType } from "../Enums";
import { IBubbleParticleData, ICoordinates, ICoordinates3d, IDelta, IHsl, IParticle, IParticleGradientAnimation, IParticleHslAnimation, IParticleLife, IParticleNumericValueAnimation, IParticleRetinaProps, IParticleRoll, IParticleTiltValueAnimation, IParticleValueAnimation, IParticleWobble, IRgb, IShapeValues } from "./Interfaces";
import { IParticlesOptions, Stroke } from "../Options";
import { Vector, Vector3d } from "./Utils";
import { Container } from "./Container";
import { Engine } from "../engine";
import { RecursivePartial } from "../Types";
/**
 * The single particle object
 * @category Core
 */
export declare class Particle implements IParticle {
    #private;
    readonly id: number;
    readonly container: Container;
    readonly group?: string | undefined;
    destroyed: boolean;
    lastPathTime: number;
    misplaced: boolean;
    spawning: boolean;
    splitCount: number;
    unbreakable: boolean;
    readonly pathDelay: number;
    readonly sides: number;
    readonly options: import("../Options").ParticlesOptions;
    readonly life: IParticleLife;
    roll?: IParticleRoll;
    wobble?: IParticleWobble;
    backColor?: IHsl;
    close: boolean;
    direction: number;
    fill: boolean;
    randomIndexData?: number;
    gradient?: IParticleGradientAnimation;
    rotate?: IParticleValueAnimation<number>;
    tilt?: IParticleTiltValueAnimation;
    color?: IParticleHslAnimation;
    opacity?: IParticleNumericValueAnimation;
    strokeWidth?: number;
    stroke?: Stroke;
    strokeColor?: IParticleHslAnimation;
    readonly moveCenter: ICoordinates & {
        radius: number;
    };
    readonly moveDecay: number;
    readonly outType: ParticleOutType;
    readonly position: Vector3d;
    readonly offset: Vector;
    readonly shadowColor: IRgb | undefined;
    readonly size: IParticleNumericValueAnimation;
    readonly velocity: Vector;
    readonly shape: string;
    readonly initialPosition: Vector;
    readonly initialVelocity: Vector;
    readonly shapeData?: IShapeValues;
    readonly bubble: IBubbleParticleData;
    readonly zIndexFactor: number;
    readonly retina: IParticleRetinaProps;
    constructor(id: number, container: Container, engine: Engine, position?: ICoordinates, overrideOptions?: RecursivePartial<IParticlesOptions>, group?: string | undefined);
    isVisible(): boolean;
    isInsideCanvas(): boolean;
    draw(delta: IDelta): void;
    getPosition(): ICoordinates3d;
    getRadius(): number;
    getMass(): number;
    getFillColor(): IHsl | undefined;
    getStrokeColor(): IHsl | undefined;
    destroy(override?: boolean): void;
    /**
     * This method is used when the particle has lost a life and needs some value resets
     */
    reset(): void;
    private split;
    private calcPosition;
    private checkOverlap;
    private calculateVelocity;
    private loadShapeData;
    private loadLife;
}
